#!/usr/bin/perl -w

# Cacti script for wifi interface data
# v0.3 - Licence : GPL
# BuggerOne @ gmail.com

my $iwconfig = "/sbin/iwconfig";
my $procfile = "/proc/net/wireless";

# Regex for every supported function
my %regex = ("bitRate",     ".*Bit Rate[:=](.*) (Mb\/s)",
             "quality",     ".*Link Quality[:=](.*)\/100",
             "signalLevel", ".*Signal level[:=](.*?) ",
             "noiseLevel",  ".*Noise level[:=](.*?) ",
             "essid",       ".*ESSID[:=][\'\"](.*)[\'\"]",
             "description", "^\\w+\\d+\\s+(.*)\\s+ESSID",
             "index",       "NoRegex"
            );

# get input parameter
my $action = $ARGV[0] if defined($ARGV[0]); # action of the script
my $fct    = $ARGV[1] if defined($ARGV[1]); # function to query
my $itf    = $ARGV[2] if defined($ARGV[2]); # Interface to get

# check command line options
if ( ( not defined ($action) ) or
     ( ($action ne "index") and ($action ne "query") and ($action ne "get") ) or
     ( ($action eq "query") and (not defined($fct) ) ) or
     ( ($action eq "get"  ) and (not (defined($fct) and defined($itf)) ) ) or
     ( (defined($fct) and not defined ($regex{$fct}) ) )
   )
{
   print "usage: $0 (index|query  <function>|get <function> <itf>)\n\tfunction: ";
   foreach (keys(%regex))
   {
      print $_,", ";
   }
   print "\n";
   exit(1);
}

## Functions ##
sub function 
{
   my $fct_regex = $regex{$_[0]};
   my $query_itf = $_[1];
   my @output = `$iwconfig $query_itf`;
   # print @output;
   
   foreach (@output)
   {
      if (m/$fct_regex/)
      {
         return $1;
      }
   }
} # function

## Main ##

# get list of wifi interfaces
open F, "< $procfile" or die "cannot open $procfile : $!";
while (<F>)
{
   if (m/\s*(\w+\d+):/)
   {
      push (@wifi,"$1");
   }
}
close F;

# do the treatment
if ($action eq "index")
{ # get interface list
   foreach(@wifi)
   {
      printf("%s\n",$_);
   }
}
elsif ($action eq "query")
{  # query all interfaces
   if ($fct eq "index")
   {
      foreach(@wifi)
      {
         printf("%s:%s\n",$_,$_);
      }
   }
   else
   {
      foreach(@wifi)
      {
         $value = function($fct,$_);
         printf ("%s:%s\n",$_,$value);
      }
   }
}
elsif ($action eq "get")
{ # query one interface
   print function ($fct,$itf),"\n";
}
else
{ # should never go here
   print "Unknown action: $action\n";
   exit(1);
}

exit(0);

__END__

